# -*- coding: utf-8 -*-

from text_cleaner.processor.misc import URL, ESCAPED_WHITESPACE


URL = URL.replace('TCURL')


def test_url():

    raw = (
        '点击http://t.cn/RtU0mZ1 查看'
    )
    expected = (
        u'点击TCURL 查看'
    )

    assert expected == URL.remove(raw)

    raw = (
        '点击http://173.255.0.1/browser.php?u=test 查看'
    )
    expected = (
        u'点击TCURL 查看'
    )

    assert expected == URL.remove(raw)


def test_escaped_whitespace():

    raw = (
        '点击\\nhttp://t.cn/RtU0mZ1 \\t查看'
    )
    expected = (
        u'点击 http://t.cn/RtU0mZ1  查看'
    )

    assert expected == ESCAPED_WHITESPACE.remove(raw)



# -*- coding: utf-8 -*-
from __future__ import (
    division, absolute_import, print_function, unicode_literals,
)
from builtins import *                  # noqa
from future.builtins.disabled import *  # noqa

import re


DEFAULT_REPLACE_TEXT = ' '


def join_regexes(regexes, delimiter='|'):
    return delimiter.join(regexes)


def merge_processors(processors, replace_text=DEFAULT_REPLACE_TEXT):
    regexes = [p.regex for p in processors]
    return RegexProcessor(
        join_regexes(regexes), replace_text,
    )


def decode(text):
    if isinstance(text, bytes):
        return text.decode('utf-8')
    else:
        return text


def force_unicode(method):

    def _wrapper(self, text):

        text = decode(text)
        return method(self, text)

    return _wrapper


class UnicodeRange(object):

    def __init__(self, begin, end):
        self.begin = int(begin)
        self.end = int(end)

    @property
    def regex(self):
        # 1. str, for formatting.
        pattern = '\\U{begin:0>8X}-\\U{end:0>8X}'.format(
            begin=self.begin, end=self.end,
        )
        # 2. to bytes.
        pattern = pattern.encode('utf-8')
        # 3. decode to str.
        return pattern.decode('unicode_escape')


class RegexProcessor(object):

    def __init__(self, regex, replace_text=DEFAULT_REPLACE_TEXT):
        # force unicode.
        self._regex = decode(regex)
        self._replace_text = decode(replace_text)

        self._regex_obj = re.compile(self.regex, re.UNICODE)

    @property
    def regex(self):
        return self._regex

    @property
    def replace_text(self):
        return self._replace_text

    def copy(self):
        return RegexProcessor(self.regex, self.replace_text)

    @force_unicode
    def replace(self, new_replace_text):
        obj = self.copy()
        obj._replace_text = new_replace_text
        return obj

    @force_unicode
    def remove(self, text):
        return self._regex_obj.sub(
            self._replace_text,
            text,
        )

    @force_unicode
    def keep(self, text):
        return self._replace_text.join(
            # force non-capture.
            m.group() for m in self._regex_obj.finditer(text),
        )

    @force_unicode
    def verify(self, text):
        match = self._regex_obj.match(text)
        if match is None:
            return False
        else:
            return match.group() == text


class UnicodeRangeProcessor(RegexProcessor):

    def __init__(self, ranges, replace_text=DEFAULT_REPLACE_TEXT):
        self._ranges = ranges

        super().__init__(
            self.ranges_to_regex(ranges),
            replace_text,
        )

    def ranges_to_regex(self, ranges):
        joined = join_regexes(
            map(lambda ur: ur.regex, ranges),
            delimiter='',
        )
        return r'[{0}]+'.format(joined)

    def copy(self):
        return UnicodeRangeProcessor(self._ranges, self.replace_text)



# -*- coding: utf-8 -*-
from __future__ import (
    division, absolute_import, print_function, unicode_literals,
)
from builtins import *                  # noqa
from future.builtins.disabled import *  # noqa

from text_cleaner.processor.processor import (
    UnicodeRange,
    UnicodeRangeProcessor,
)
from text_cleaner.processor.common import (
    SYMBOLS_AND_PUNCTUATION_EXTENSION_RANGES,
)


# https://en.wikipedia.org/wiki/CJK_Unified_Ideographs

CJK_COMMON_RANGES = [
    UnicodeRange(0x4E00, 0x9FFF),
]

CJK_EXTENSION_RANGES = [
    UnicodeRange(0x3400, 0x4DFF),
    UnicodeRange(0x20000, 0x2A6DF),
    UnicodeRange(0x2A700, 0x2B73F),
    UnicodeRange(0x2B740, 0x2B81F),
    UnicodeRange(0x2B820, 0x2CEAF),
]

CJK_COMPATIBILITY_RANGES = [
    UnicodeRange(0x3300, 0x33FF),
    UnicodeRange(0xFE30, 0xFE4F),
    UnicodeRange(0xF900, 0xFAFF),
    UnicodeRange(0x2F800, 0x2FA1F),
]

# http://www.unicode.org/charts/PDF/U3000.pdf
# http://www.unicode.org/charts/PDF/UFF00.pdf

CJK_SYMBOLS_AND_PUNCTUATION_RANGES = [
    UnicodeRange(0x3000, 0x303F),
]


# processors.

CHINESE_EXTENSION = UnicodeRangeProcessor(
    CJK_EXTENSION_RANGES,
)

CHINESE_COMPATIBILITY = UnicodeRangeProcessor(
    CJK_COMPATIBILITY_RANGES,
)

CHINESE_SYMBOLS_AND_PUNCTUATION_RANGES = UnicodeRangeProcessor(
    CJK_SYMBOLS_AND_PUNCTUATION_RANGES,
)

# common usage.
CHINESE_CHARACTER = UnicodeRangeProcessor(
    CJK_COMMON_RANGES,
)

CHINESE = UnicodeRangeProcessor(
    CJK_COMMON_RANGES +
    CJK_SYMBOLS_AND_PUNCTUATION_RANGES +
    SYMBOLS_AND_PUNCTUATION_EXTENSION_RANGES,
)

# full version.
CHINESE_ALL = UnicodeRangeProcessor(
    CJK_COMMON_RANGES +
    CJK_EXTENSION_RANGES +
    CJK_COMPATIBILITY_RANGES +
    CJK_SYMBOLS_AND_PUNCTUATION_RANGES +
    SYMBOLS_AND_PUNCTUATION_EXTENSION_RANGES,
)



# -*- coding: utf-8 -*-

from text_cleaner.processor.common import ASCII
from text_cleaner.processor.chinese import CHINESE
from text_cleaner.processor.misc import URL

from text_cleaner import remove, keep


def test_remove():
    raw = (
        '点击http://t.cn/RtU0mZ1 查看,123456,test'
    )
    expected = (
        u'点击  查看 '
    )

    assert expected == remove(raw, [URL, ASCII])


def test_keep():
    raw = (
        '点击http://t.cn/RtU0mZ1 查看,123456,test'
    )
    expected = (
        u'点击 查看'
    )

    assert expected == keep(raw, [CHINESE])


def test_keep_with_cache():
    raw = (
        '点击http://t.cn/RtU0mZ1 查看,123456,test'
    )
    expected = (
        u'点击 http://t.cn/RtU0mZ1 查看'
    )

    assert expected == keep(raw, [CHINESE, URL])
    assert expected == keep(raw, [CHINESE, URL])



# -*- coding: utf-8 -*-
from __future__ import (
    division, absolute_import, print_function, unicode_literals,
)
from builtins import *                  # noqa
from future.builtins.disabled import *  # noqa

from text_cleaner.processor.processor import merge_processors


__all__ = [
    b'remove',
    b'keep',
]


_CACHED_MERGED_PROCESSORS = {}


def remove(text, processors):
    for processor in processors:
        text = processor.remove(text)
    return text


def keep(text, processors):
    key = tuple(processors)
    if key not in _CACHED_MERGED_PROCESSORS:
        _CACHED_MERGED_PROCESSORS[key] = merge_processors(key)

    processor = _CACHED_MERGED_PROCESSORS[key]
    return processor.keep(text)



# -*- coding: utf-8 -*-
from __future__ import (
    division, absolute_import, print_function, unicode_literals,
)
from builtins import *                  # noqa
from future.builtins.disabled import *  # noqa

from text_cleaner.processor.processor import (
    RegexProcessor,
)


URL = RegexProcessor(
    # https://mathiasbynens.be/demo/url-regex
    # https://gist.github.com/dperini/729294

    # protocol identifier
    "(?:(?:https?|ftp)://)"
    # user:pass authentication
    "(?:\S+(?::\S*)?@)?"
    "(?:"
    # IP address exclusion
    # private & local networks
    "(?!(?:10|127)(?:\.\d{1,3}){3})"
    "(?!(?:169\.254|192\.168)(?:\.\d{1,3}){2})"
    "(?!172\.(?:1[6-9]|2\d|3[0-1])(?:\.\d{1,3}){2})"
    # IP address dotted notation octets
    # excludes loopback network 0.0.0.0
    # excludes reserved space >= 224.0.0.0
    # excludes network & broadcast addresses
    # (first & last IP address of each class)
    "(?:[1-9]\d?|1\d\d|2[01]\d|22[0-3])"
    "(?:\.(?:1?\d{1,2}|2[0-4]\d|25[0-5])){2}"
    "(?:\.(?:[1-9]\d?|1\d\d|2[0-4]\d|25[0-4]))"
    "|"
    # host name
    "(?:(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)"
    # domain name
    "(?:\.(?:[a-z\u00a1-\uffff0-9]-?)*[a-z\u00a1-\uffff0-9]+)*"
    # TLD identifier
    "(?:\.(?:[a-z\u00a1-\uffff]{2,}))"
    ")"
    # port number
    "(?::\d{2,5})?"
    # resource path
    "(?:/\S*)?"
)


ESCAPED_WHITESPACE = RegexProcessor(
    r'[\\][ tnrfv]'
)



# -*- coding: utf-8 -*-
from __future__ import (
    division, absolute_import, print_function, unicode_literals,
)
from builtins import *                  # noqa
from future.builtins.disabled import *  # noqa

# remove following code.
from text_cleaner.main import entry_point


def test_entry_point():
    assert 42 == entry_point()



# -*- coding: utf-8 -*-
from __future__ import (
    division, absolute_import, print_function, unicode_literals,
)
from builtins import *                  # noqa
from future.builtins.disabled import *  # noqa

from text_cleaner.processor.processor import (
    UnicodeRange,
    UnicodeRangeProcessor,
)


# basic.

ASCII_ALPHA_RANGES = [
    UnicodeRange(0x0041, 0x005A),
    UnicodeRange(0x0061, 0x007A),
]

ASCII_DIGIT_RANGES = [
    UnicodeRange(0x0030, 0x0039),
]

ASCII_SYMBOLS_AND_PUNCTUATION_RANGES = [
    UnicodeRange(0x0021, 0x002F),
    UnicodeRange(0x003A, 0x0040),
    UnicodeRange(0x005B, 0x0060),
    UnicodeRange(0x007B, 0x007E),
]


# extension.

ALPHA_EXTENSION_RANGES = [
    UnicodeRange(0xFF21, 0xFF3A),
    UnicodeRange(0xFF41, 0xFF5A),
]

DIGIT_EXTENSION_RANGES = [
    UnicodeRange(0xFF10, 0xFF19),
]

SYMBOLS_AND_PUNCTUATION_EXTENSION_RANGES = [
    UnicodeRange(0xFF00, 0xFF0F),
    UnicodeRange(0xFF1A, 0xFF20),
    UnicodeRange(0xFF3B, 0xFF40),
    UnicodeRange(0xFF5B, 0xFF64),
    UnicodeRange(0xFFE0, 0xFFEE),
]


# processor.
ALPHA = UnicodeRangeProcessor(
    ASCII_ALPHA_RANGES,
)
DIGIT = UnicodeRangeProcessor(
    ASCII_DIGIT_RANGES,
)
SYMBOLS_AND_PUNCTUATION = UnicodeRangeProcessor(
    ASCII_SYMBOLS_AND_PUNCTUATION_RANGES,
)
ASCII = UnicodeRangeProcessor(
    ASCII_ALPHA_RANGES +
    ASCII_DIGIT_RANGES +
    ASCII_SYMBOLS_AND_PUNCTUATION_RANGES,
)


ALPHA_EXTENSION = UnicodeRangeProcessor(
    ASCII_ALPHA_RANGES +
    ALPHA_EXTENSION_RANGES,
)
DIGIT_EXTENSION = UnicodeRangeProcessor(
    ASCII_DIGIT_RANGES +
    DIGIT_EXTENSION_RANGES,
)
SYMBOLS_AND_PUNCTUATION_EXTENSION = UnicodeRangeProcessor(
    ASCII_SYMBOLS_AND_PUNCTUATION_RANGES +
    SYMBOLS_AND_PUNCTUATION_EXTENSION_RANGES,
)



# -*- coding: utf-8 -*-

from text_cleaner.processor.processor import merge_processors
from text_cleaner.processor.common import ASCII
from text_cleaner.processor.chinese import CHINESE, CHINESE_CHARACTER


CHINESE_WITH_ASCII = merge_processors(
    [CHINESE, ASCII],
    replace_text='',
)


def test_demo():
    assert u'中文测试' == CHINESE.keep('中文测试')
    assert u'中文测试' == CHINESE.keep(u'中文测试')


def test_keep():
    raw = (
        u'NASA中文，由天文爱好者更新维护。'
        u'微信号：nasawatch。NASA中文的微博主页、个人资料、相册。'
        u'新浪微博，随时随地分享身边的新鲜事儿。'
    )
    expected = (
        u'中文，由天文爱好者更新维护。'
        u'微信号： 。 中文的微博主页、个人资料、相册。'
        u'新浪微博，随时随地分享身边的新鲜事儿。'
    )

    assert expected == CHINESE.keep(raw)
    assert raw == CHINESE_WITH_ASCII.keep(raw)

    expected = (
        u'中文 由天文爱好者更新维护 '
        u'微信号 中文的微博主页 个人资料 相册 '
        u'新浪微博 随时随地分享身边的新鲜事儿'
    )
    assert expected == CHINESE_CHARACTER.keep(raw)


def test_verify():
    assert CHINESE_CHARACTER.verify('中文')
    assert CHINESE_CHARACTER.verify(u'中文')
    assert not CHINESE_CHARACTER.verify('中文1')
    assert not CHINESE_CHARACTER.verify(u'中文word')
    assert not CHINESE_CHARACTER.verify(u'word中文')
